@@ -37,6 +37,9 @@ GVirConfigDomainDevice *
 gvir_config_domain_disk_new_from_tree(GVirConfigXmlDoc *doc,
                                       xmlNodePtr tree);
 GVirConfigDomainDevice *
+gvir_config_domain_filesys_new_from_tree(GVirConfigXmlDoc *doc,
+                                         xmlNodePtr tree);
+GVirConfigDomainDevice *
 gvir_config_domain_graphics_new_from_tree(GVirConfigXmlDoc *doc,
                                           xmlNodePtr tree);
 GVirConfigDomainDevice *
