@@ -355,18 +355,20 @@ stream_sink(virStreamPtr st G_GNUC_UNUSED,
  * @cancellable: cancellation notifier
  * @func: (scope notified): the callback for writing data to application
  * @user_data: (closure): data to be passed to @callback
- * Returns: the number of bytes consumed or -1 upon error
+ * @error: #GError for error reporting, or %NULL to ignore.
  *
  * Receive the entire data stream, sending the data to the
  * requested data sink. This is simply a convenient alternative
  * to virStreamRecv, for apps that do blocking-I/o.
+ *
+ * Returns: the number of bytes consumed or -1 upon error
  */
 gssize
 gvir_stream_receive_all(GVirStream *self,
                         GCancellable *cancellable,
                         GVirStreamSinkFunc func,
                         gpointer user_data,
-                        GError **err)
+                        GError **error)
 {
     struct stream_sink_helper helper = {
         .self = self,
@@ -377,11 +379,13 @@ gvir_stream_receive_all(GVirStream *self,
     int r;
 
     g_return_val_if_fail(GVIR_IS_STREAM(self), -1);
+    g_return_val_if_fail((cancellable == NULL) || G_IS_CANCELLABLE(cancellable), -1);
     g_return_val_if_fail(func != NULL, -1);
+    g_return_val_if_fail(error == NULL || *error == NULL, -1);
 
     r = virStreamRecvAll(self->priv->handle, stream_sink, &helper);
     if (r < 0) {
-        gvir_set_error_literal(err, GVIR_STREAM_ERROR,
+        gvir_set_error_literal(error, GVIR_STREAM_ERROR,
                                0,
                                "Unable to perform RecvAll");
     }
@@ -422,6 +426,8 @@ gssize gvir_stream_send(GVirStream *self,
 
     g_return_val_if_fail(GVIR_IS_STREAM(self), -1);
     g_return_val_if_fail(buffer != NULL, -1);
+    g_return_val_if_fail((cancellable == NULL) || G_IS_CANCELLABLE(cancellable), -1);
+    g_return_val_if_fail(error == NULL || *error == NULL, -1);
 
     if (g_cancellable_set_error_if_cancelled (cancellable, error))
         return -1;
@@ -465,18 +471,20 @@ stream_source(virStreamPtr st G_GNUC_UNUSED,
  * @cancellable: cancellation notifier
  * @func: (scope notified): the callback for writing data to application
  * @user_data: (closure): data to be passed to @callback
- * Returns: the number of bytes consumed or -1 upon error
+ * @error: #GError for error reporting, or %NULL to ignore.
  *
  * Send the entire data stream, sending the data to the
  * requested data source. This is simply a convenient alternative
  * to virStreamRecv, for apps that do blocking-I/o.
+ *
+ * Returns: the number of bytes consumed or -1 upon error
  */
 gssize
 gvir_stream_send_all(GVirStream *self,
                      GCancellable *cancellable,
                      GVirStreamSourceFunc func,
                      gpointer user_data,
-                     GError **err)
+                     GError **error)
 {
     struct stream_source_helper helper = {
         .self = self,
@@ -487,11 +495,13 @@ gvir_stream_send_all(GVirStream *self,
     int r;
 
     g_return_val_if_fail(GVIR_IS_STREAM(self), -1);
+    g_return_val_if_fail((cancellable == NULL) || G_IS_CANCELLABLE(cancellable), -1);
     g_return_val_if_fail(func != NULL, -1);
+    g_return_val_if_fail(error == NULL || *error == NULL, -1);
 
     r = virStreamSendAll(self->priv->handle, stream_source, &helper);
     if (r < 0) {
-        gvir_set_error_literal(err, GVIR_STREAM_ERROR,
+        gvir_set_error_literal(error, GVIR_STREAM_ERROR,
                                0,
                                "Unable to perform SendAll");
     }
@@ -618,7 +628,7 @@ GSourceFuncs gvir_stream_source_funcs = {
 
 
 /**
- * gvir_stream_add_watch: (skip):
+ * gvir_stream_add_watch: (skip)
  * @stream: the stream
  * @cond: the conditions to watch for (bitfield of #GVirStreamIOCondition)
  * @func: (closure opaque): the function to call when the condition is satisfied
